/*
 * Copyright (c) 2014 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Declares that the object referenced by the local variable, return value,
 * receiver, or formal
 * parameter is only partially initialized. That is, some of the fields of the
 * class may hold <code>null</code> values even though the field is annotated to
 * be {@link NonNull}. Furthermore, the local variable, return value, receiver,
 * or formal
 * parameter is itself non-<code>null</code>.
 * 
 * <p>
 * The extent to which the referenced object is initialized is described by the
 * {@link #upTo() upTo} attribute.
 * 
 * <p>When the annotation appears on a method, the {@link #value() value} attribute
 * is used to distinguish whether the return value or receiver is being 
 * annotated.
 *
 * <p>Raw references originate in constructors: an object of type <code>C</code>
 * is not fully initialized until its final constructor (the one for <code>C</code>)
 * executes.  If a reference to the receiver is passed to another method
 * during construction, then that reference is raw. 
 * 
 * <h3>Semantics:</h3>
 * 
 * <i>Local variable:</i> The local variable will never be <code>null</code>
 * and always refers to an object that is at least as initialized as the given
 * {@link #upTo() upTo} attribute.
 * <p>
 * 
 * <i>Parameter:</i> At the start of the method's execution, the annotated
 * parameter may refer to an object that is not fully initialized.  The parameter 
 * is never <code>null</code>.
 * <p>
 * <i>Return Value:</i> The value returned by the annotated method refers to
 * an object that is not fully initialized.  The return value is not
 * <code>null</code>.
 * <p>
 * <i>Receiver:</i> For a method, the object the method was invoked upon, may
 * not be fully initialized.  
 * <p>
 * 
 * <h3>Examples:</h3>
 * 
 * The constructor for <code>A</code> uses the object
 * under construction as the receiver for the method <code>m()</code>.  Thus 
 * method <code>m()</code> cannot assume the receiver is fully initialized.  
 * In particular, the field <code>path</code> in <code>B.m()</code> cannot
 * be assumed to be non-<code>null</code> even though it is annotated as 
 * {@link NonNull}.
 * 
 * <pre>
 * class A {
 *   &#064;NonNull
 *   protected String name;
 *   
 *   public A(&#064;NonNull String s) {
 *     this.name = s;
 *     this.m(55);
 *   }
 *   
 *   &#064;Raw(upTo="java.lang.Object", value="this")
 *   protected void m(int x) { &hellip; }
 * }  
 * 
 * class B extends A {
 *   &#064;NonNull
 *   protected String path;
 *   
 *   public B(&#064;NonNull String p, &#064;NonNull String s) {
 *     super(s);
 *     path = p;
 *   }
 *   
 *   &#064;Raw(upTo="java.lang.Object", value="this")
 *   protected void m(int x) {
 *     &hellip; this.path &hellip; // Cannot assume that "this.path" is non-null
 *   }
 * }
 * </pre>
 * 
 * @see NonNull
 * @see Nullable
 * @see Raws
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.METHOD, ElementType.CONSTRUCTOR, ElementType.PARAMETER,
  ElementType.LOCAL_VARIABLE })
public @interface Raw {
  /**
   * Describes the extent to which the referenced object is considered
   * initialized.
   * <dl>
   * <dt>upTo="*"
   * <dd>The object must be assumed to be completely uninitialized. Any
   * {@link NonNull} field of the object may in fact be <code>null</code>. This is
   * the default value of the {@link #upTo upTo} attribute, and there is never any
   * reason to state it explicitly. That is, <code>&#064;Raw</code> is preferred
   * over <code>&#064;Raw(upTo="*")</code>.
   * 
   * <dt>upTo="<i>qualified-type-name</i>"
   * <dd>The object may be assumed to initialized up to the named ancestor type.
   * That is, the constructors for each ancestor type from
   * <code>java.lang.Object</code> down to and including the named type have been
   * successfully invoked. Any {@link NonNull} field in the object declared in
   * types that are descendants of the named type may in fact be <code>null</code>.
   * </dl>
   */
  String upTo() default "*";

  /**
   * When annotating a method, this attribute is used to disambiguate whether
   * the annotation refers to the method's receiver, the method's return value,
   * or both. The value is comma separated list of tokens, and has the following
   * set of legal values (ignoring white space issues):
   * <ul>
   * <li>{@code ""}
   * <li>{@code "this"}
   * <li>{@code "return"}
   * <li>{@code "this, return"}
   * <li>{@code "return, this"}
   * </ul>
   * 
   * <p>
   * The values are interpreted thusly
   * <ul>
   * <li>If the list contains the value {@code "this"}, it indicates the
   * receiver is unique. This value is only allowed on methods.
   * <li>If the list contains the value {@code "return"}, it indicates the
   * return value is unique. This value is allowed on methods and constructors.
   * <li>If the list contains both {@code "this"} and {@code "return"}, it
   * indicates that both the receiver and the return value are unique. This
   * value is only allowed on methods.
   * </ul>
   * <p>
   * This attribute is not used when annotating a parameter or a field: the
   * attribute value must be the empty string in these cases.
   * <p>
   * The value of this attribute must conform to the following grammar (in <a
   * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
   * Form</a>):
   * 
   * <pre>
   * value = [(&quot;this&quot; [&quot;,&quot; &quot;return&quot;]) / (&quot;return&quot; [&quot;,&quot; &quot;this&quot;])] ; See above comments
   * </pre>
   */
  String value() default "";
}
